module ServiceLayerRest
  class ApplicationController < ActionController::Base
    include Constants
    include ResponseHelper
    protect_from_forgery with: :exception

    before_action :check_auth, :except => [:login, :get_clients]
    before_action :check_current_user, :except => [:login, :get_clients]

    def check_auth
      ServiceLayerDll.init

      authenticate_or_request_with_http_basic do |username, password|
        header_params = check_header_params

        unless header_params.empty?
          response_string, string_pointer = ServiceLayerDll.check_auth(username, password, header_params[Constants::AUTHTYPE_KEY], header_params[Constants::MANDANT_ID_KEY].to_i)
          ServiceLayerDll.free_string(string_pointer)

          response = Oj.load(response_string)
          auth_params = check_response_params(response)

          unless auth_params.empty?
            user_data = auth_params[Constants::USERDATA_KEY]

            user = User.new(user_data)
            user.id = user_data[Constants::ID_CLIENT_KEY]

            sign_in :user, user
          end
        end
      end
    end

    def check_current_user
      return render bad_params_for_render('anwender') if current_user.nil?
    end

    private
    def check_header_params
      # Check request/response
      return {} if request.nil?

      # Check authtype
      return {} if request.headers[Constants::AUTHTYPE_KEY].nil? || request.headers[Constants::AUTHTYPE_KEY].blank?

      # Check mandant id
      return {} if request.headers[Constants::MANDANTID_KEY].nil? || request.headers[Constants::MANDANTID_KEY].blank?

      {
          Constants::AUTHTYPE_KEY => request.headers[Constants::AUTHTYPE_KEY],
          Constants::MANDANT_ID_KEY => request.headers[Constants::MANDANTID_KEY]
      }
    end

    def check_response_params(response)
      # Check request/response
      return {} if response.nil?

      # Check status
      return {} if response[Constants::STATUS_KEY].nil? || response[Constants::STATUS_KEY] != Rack::Utils::SYMBOL_TO_STATUS_CODE[:ok]

      # Check data/user
      return {} if response[Constants::DATA_KEY].nil? || !response[Constants::DATA_KEY].is_a?(Hash) || response[Constants::DATA_KEY].empty?

      {
          Constants::USERDATA_KEY => response[Constants::DATA_KEY]
      }
    end
  end
end
