module ServiceLayerRest
  class StaffSchedulingsController < ApplicationController
    include Base
    include Dll

    #TODO: remove time_... stuff

    def get_daily_data_all
      invalid_key = check_params([:fct_code, :from, :to], [:planarea_id, :employee_id])
      return render bad_params_for_render(invalid_key) unless invalid_key.blank?

      time_marker_all = start_timer
      begin
        time_marker_dll = start_timer
        response_string, string_pointer = ServiceLayerDll.get_staff_scheduling_daily_data_all(
          current_user.as_json,
          params[:fct_code],
          params[:planarea_id].to_i,
          params[:employee_id].to_i,
          params[:from],
          params[:to]
        )
        ServiceLayerDll.free_string(string_pointer)
        stop_timer(time_marker_dll, "Rails Duration (Dll)")

        time_marker_render = start_timer
        render parse_response_string_for_render(response_string.force_encoding("utf-8"))
        stop_timer(time_marker_render, "Rails Duration (Render)")
      rescue Exception => e
        time_marker_render = start_timer
        render parse_exception_for_render(e)
        stop_timer(time_marker_render, "Rails Duration (Render)")
      end
      stop_timer(time_marker_all, "Rails Duration (All)")
    end

    def create_daily_data_details
      create_update_daily_data_details
    end

    def update_daily_data_details
      create_update_daily_data_details
    end

    def delete_daily_data_details
      invalid_key = check_params([:fct_code, :date, :tks_timestamp], [:planarea_id, :employee_id, :tel_id, :tel_det_id])
      return render bad_params_for_render(invalid_key) unless invalid_key.blank?

      begin
        response_string, string_pointer = ServiceLayerDll.delete_staff_scheduling_daily_data_details(
          current_user.as_json,
          params[:fct_code],
          params[:planarea_id].to_i,
          params[:employee_id].to_i,
          params[:tel_id].to_i,
          params[:tel_det_id].to_i,
          params[:date],
          params[:tks_timestamp]
        )
        ServiceLayerDll.free_string(string_pointer)

        render parse_response_string_for_render(response_string.force_encoding("utf-8"))
      rescue Exception => e
        render parse_exception_for_render(e)
      end
    end

    def get_employee_data
      invalid_key = check_params([:fct_code, :from, :to], [:planarea_id, :employee_id])
      return render bad_params_for_render(invalid_key) unless invalid_key.blank?

      begin
        response_string, string_pointer = ServiceLayerDll.get_staff_scheduling_employee_data(
          current_user.as_json,
          params[:fct_code],
          params[:planarea_id].to_i,
          params[:employee_id].to_i,
          params[:from],
          params[:to]
        )
        ServiceLayerDll.free_string(string_pointer)

        render parse_response_string_for_render(response_string.force_encoding("utf-8"))
      rescue Exception => e
        render parse_exception_for_render(e)
      end
    end

    def get_employee_planareas
        invalid_key = check_params([:fct_code, :from, :to], [:employee_id])
        return render bad_params_for_render(invalid_key) unless invalid_key.blank?
  
        begin
          response_string, string_pointer = ServiceLayerDll.get_staff_scheduling_employee_planareas(
            current_user.as_json,
            params[:fct_code],
            params[:employee_id].to_i,
            params[:from],
            params[:to]
          )
          ServiceLayerDll.free_string(string_pointer)
  
          render parse_response_string_for_render(response_string.force_encoding("utf-8"))
        rescue Exception => e
          render parse_exception_for_render(e)
        end
    end

    def get_contract_data
      invalid_key = check_params([:fct_code, :from, :to], [:planarea_id, :employee_id])
      return render bad_params_for_render(invalid_key) unless invalid_key.blank?

      begin
        response_string, string_pointer = ServiceLayerDll.get_staff_scheduling_contract_data(
          current_user.as_json,
          params[:fct_code],
          params[:planarea_id].to_i,
          params[:employee_id].to_i,
          params[:from],
          params[:to]
        )
        ServiceLayerDll.free_string(string_pointer)

        render parse_response_string_for_render(response_string.force_encoding("utf-8"))
      rescue Exception => e
        render parse_exception_for_render(e)
      end
    end

    def get_rental_data
      invalid_key = check_params([:fct_code, :from, :to], [:planarea_id, :employee_id])
      return render bad_params_for_render(invalid_key) unless invalid_key.blank?

      begin
        response_string, string_pointer = ServiceLayerDll.get_staff_scheduling_rental_data(
          current_user.as_json,
          params[:fct_code],
          params[:planarea_id].to_i,
          params[:employee_id].to_i,
          params[:from],
          params[:to]
        )
        ServiceLayerDll.free_string(string_pointer)

        render parse_response_string_for_render(response_string.force_encoding("utf-8"))
      rescue Exception => e
        render parse_exception_for_render(e)
      end
    end

    def get_surcharge_data
      invalid_key = check_params([:fct_code], [])
      return render bad_params_for_render(invalid_key) unless invalid_key.blank?

      begin
        response_string, string_pointer = ServiceLayerDll.get_staff_scheduling_surcharge_data(
          current_user.as_json,
          params[:fct_code]
        )
        ServiceLayerDll.free_string(string_pointer)

        render parse_response_string_for_render(response_string)
      rescue Exception => e
        render parse_exception_for_render(e)
      end
    end

    def get_longterm_account
      invalid_key = check_params([:fct_code, :date], [])
      return render bad_params_for_render(invalid_key) unless invalid_key.blank?

      fake_longterm_data = true
      date = DateTime.strptime(params[:date], Constants::DATE_FORMAT)

      if fake_longterm_data
          return render_fake_longterm_data(date)
      end

      begin
        response_string, string_pointer = ServiceLayerDll.get_staff_scheduling_longterm_account(
          current_user.as_json,
          params[:fct_code]
        )
        ServiceLayerDll.free_string(string_pointer)

        render parse_response_string_for_render(response_string)
      rescue Exception => e
        render parse_exception_for_render(e)
      end
    end

    def get_vacation_account
        invalid_key = check_params([:fct_code], [:planarea_id, :employee_id, :year])
        return render bad_params_for_render(invalid_key) unless invalid_key.blank?

        begin
            response_string, string_pointer = ServiceLayerDll.get_staff_scheduling_vacation_account(
                current_user.as_json,
                params[:fct_code],
                params[:planarea_id].to_i,
                params[:employee_id].to_i,
                params[:year].to_i
            )
            ServiceLayerDll.free_string(string_pointer)

            render parse_response_string_for_render(response_string)
        rescue Exception => e
            render parse_exception_for_render(e)
        end
    end

    def get_annual_working_time_account
      invalid_key = check_params([:fct_code], [:year])
      return render bad_params_for_render(invalid_key) unless invalid_key.blank?

      begin
        response_string, string_pointer = ServiceLayerDll.get_staff_scheduling_annual_working_time_account(
          current_user.as_json,
          params[:fct_code]
        )
        ServiceLayerDll.free_string(string_pointer)

        render parse_response_string_for_render(response_string)
      rescue Exception => e
        render parse_exception_for_render(e)
      end
    end

    def get_timesheet
      invalid_key = check_params([:fct_code], [])
      return render bad_params_for_render(invalid_key) unless invalid_key.blank?

      begin
        response_string, string_pointer = ServiceLayerDll.get_staff_scheduling_timesheet(
          current_user.as_json,
          params[:fct_code]
        )
        ServiceLayerDll.free_string(string_pointer)

        render parse_response_string_for_render(response_string)
      rescue Exception => e
        render parse_exception_for_render(e)
      end
    end

    def update_timesheet
      invalid_key = check_params([:fct_code], [])
      return render bad_params_for_render(invalid_key) unless invalid_key.blank?

      begin
        response_string, string_pointer = ServiceLayerDll.update_staff_scheduling_timesheet(
          current_user.as_json,
          params[:fct_code]
        )
        ServiceLayerDll.free_string(string_pointer)

        render parse_response_string_for_render(response_string)
      rescue Exception => e
        render parse_exception_for_render(e)
      end
    end

    def create_timesheet
      invalid_key = check_params([:fct_code], [])
      return render bad_params_for_render(invalid_key) unless invalid_key.blank?

      begin
        response_string, string_pointer = ServiceLayerDll.create_staff_scheduling_timesheet(
          current_user.as_json,
          params[:fct_code]
        )
        ServiceLayerDll.free_string(string_pointer)

        render parse_response_string_for_render(response_string)
      rescue Exception => e
        render parse_exception_for_render(e)
      end
    end

    def delete_timesheet
      invalid_key = check_params([:fct_code], [])
      return render bad_params_for_render(invalid_key) unless invalid_key.blank?

      begin
        response_string, string_pointer = ServiceLayerDll.delete_staff_scheduling_timesheet(
          current_user.as_json,
          params[:fct_code]
        )
        ServiceLayerDll.free_string(string_pointer)

        render parse_response_string_for_render(response_string)
      rescue Exception => e
        render parse_exception_for_render(e)
      end
    end

    def get_release_working_times
      invalid_key = check_params([:fct_code], [])
      return render bad_params_for_render(invalid_key) unless invalid_key.blank?

      begin
        response_string, string_pointer = ServiceLayerDll.get_staff_scheduling_release_working_times(
          current_user.as_json,
          params[:fct_code]
        )
        ServiceLayerDll.free_string(string_pointer)

        render parse_response_string_for_render(response_string)
      rescue Exception => e
        render parse_exception_for_render(e)
      end
    end

    def create_release_working_times
      invalid_key = check_params([:fct_code], [])
      return render bad_params_for_render(invalid_key) unless invalid_key.blank?

      begin
        response_string, string_pointer = ServiceLayerDll.create_staff_scheduling_release_working_times(
          current_user.as_json,
          params[:fct_code]
        )
        ServiceLayerDll.free_string(string_pointer)

        render parse_response_string_for_render(response_string)
      rescue Exception => e
        render parse_exception_for_render(e)
      end
    end

    def get_working_time_entries
      invalid_key = check_params([:fct_code], [])
      return render bad_params_for_render(invalid_key) unless invalid_key.blank?

      begin
        response_string, string_pointer = ServiceLayerDll.get_staff_scheduling_working_time_entries(
          current_user.as_json,
          params[:fct_code]
        )
        ServiceLayerDll.free_string(string_pointer)

        render parse_response_string_for_render(response_string)
      rescue Exception => e
        render parse_exception_for_render(e)
      end
    end

    def create_working_time_entries
      invalid_key = check_params([:fct_code], [])
      return render bad_params_for_render(invalid_key) unless invalid_key.blank?

      begin
        response_string, string_pointer = ServiceLayerDll.create_staff_scheduling_working_time_entries(
          current_user.as_json,
          params[:fct_code]
        )
        ServiceLayerDll.free_string(string_pointer)

        render parse_response_string_for_render(response_string)
      rescue Exception => e
        render parse_exception_for_render(e)
      end
    end

    def update_month_confirmation
      invalid_key = check_params([:fct_code, :from, :to], [:plan_id, :status, :planarea_id, :employee_id])
      return render bad_params_for_render(invalid_key) unless invalid_key.blank?

      begin
        response_string, string_pointer = ServiceLayerDll.update_month_confirmation(
          current_user.as_json,
          params[:fct_code],
          params[:plan_id].to_i,
          params[:status].to_i,
          params[:planarea_id].to_i,
          params[:employee_id].to_i,
          params[:from],
          params[:to]
        )
        ServiceLayerDll.free_string(string_pointer)
        
        render parse_response_string_for_render(response_string)
      rescue Exception => e
        render parse_exception_for_render(e)
      end
    end

    def get_pinboard
        invalid_key = check_params([:fct_code, :date], [:planarea_id])
        return render bad_params_for_render(invalid_key) unless invalid_key.blank?
  
        begin
          response_string, string_pointer = ServiceLayerDll.get_pinboard(
            current_user.as_json,
            params[:fct_code],
            params[:planarea_id].to_i,
            params[:date]
          )
          ServiceLayerDll.free_string(string_pointer)
        
          render parse_response_string_for_render(response_string)
        rescue Exception => e
          render parse_exception_for_render(e)
        end
    end

    def get_employee_assignment_period
        invalid_key = check_params([:fct_code], [:employee_id])
        return render bad_params_for_render(invalid_key) unless invalid_key.blank?
  
        begin
          response_string, string_pointer = ServiceLayerDll.get_employee_assignment_period(
            current_user.as_json,
            params[:fct_code],
            params[:employee_id].to_i
          )
          ServiceLayerDll.free_string(string_pointer)

          render parse_response_string_for_render(response_string)
        rescue Exception => e
          render parse_exception_for_render(e)
        end
    end

    private

    def create_update_daily_data_details
      invalid_key = check_params([:fct_code], [])
      return render bad_params_for_render(invalid_key) unless invalid_key.blank?

      checked_params = check_params_for_tlist(params, :change_tel_det_list, "zu ändernden TELDET-Details")
      return render checked_params unless checked_params == true

      begin
        response_string, string_pointer = ServiceLayerDll.create_update_staff_scheduling_daily_data_details(
          current_user.as_json,
          params[:fct_code],
          params[:change_tel_det_list].to_json
        )
        ServiceLayerDll.free_string(string_pointer)

        render parse_response_string_for_render(response_string.force_encoding("utf-8"))
      rescue Exception => e
        render parse_exception_for_render(e)
      end
    end

    def render_fake_longterm_data(date)
        account_data = [
                {
                    :date => "2017-01-04T00:00:00.000+00:00",
                    :hours => random_fake_longterm_hours(),
                    :comment => "???",
                    :created_at => "2018-07-18T14:38:00.000+00:00",
                    :created_by => "admin",
                    :updated_at => "",
                    :updated_by => "admin"
                },

                {
                    :date => "2018-02-15T00:00:00.000+00:00",
                    :hours => random_fake_longterm_hours(),
                    :comment => "",
                    :created_at => "2018-07-18T14:40:00.000+00:00",
                    :created_by => "admin",
                    :updated_at => "2018-07-19T10:32:00.000+00:00",
                    :updated_by => "admin"
                },

                {
                    :date => "2018-07-01T00:00:00.000+00:00",
                    :hours => random_fake_longterm_hours(),
                    :comment => "",
                    :created_at => "2018-07-19T08:02:00.000+00:00",
                    :created_by => "admin",
                    :updated_at => "",
                    :updated_by => "admin"
                },

                {
                    :date => "2018-07-02T00:00:00.000+00:00",
                    :hours => random_fake_longterm_hours(),
                    :comment => "",
                    :created_at => "2018-07-19T08:04:00.000+00:00",
                    :created_by => "admin",
                    :updated_at => "",
                    :updated_by => "admin"
                },

                {
                    :date => "2018-07-03T00:00:00.000+00:00",
                    :hours => random_fake_longterm_hours(),
                    :comment => "",
                    :created_at => "2018-07-19T08:04:00.000+00:00",
                    :created_by => "admin",
                    :updated_at => "",
                    :updated_by => "admin"
                },

                {
                    :date => "2018-07-04T00:00:00.000+00:00",
                    :hours => random_fake_longterm_hours(),
                    :comment => "",
                    :created_at => "2018-07-19T08:04:00.000+00:00",
                    :created_by => "admin",
                    :updated_at => "",
                    :updated_by => "admin"
                },

                {
                    :date => "2018-07-05T00:00:00.000+00:00",
                    :hours => random_fake_longterm_hours(),
                    :comment => "",
                    :created_at => "2018-07-19T08:04:00.000+00:00",
                    :created_by => "admin",
                    :updated_at => "",
                    :updated_by => "admin"
                },

                {
                    :date => "2018-07-06T00:00:00.000+00:00",
                    :hours => random_fake_longterm_hours(),
                    :comment => "",
                    :created_at => "2018-07-19T08:04:00.000+00:00",
                    :created_by => "admin",
                    :updated_at => "",
                    :updated_by => "admin"
                },

                {
                    :date => "2018-07-06T00:00:00.000+00:00",
                    :hours => random_fake_longterm_hours(),
                    :comment => "",
                    :created_at => "2018-07-19T08:04:00.000+00:00",
                    :created_by => "admin",
                    :updated_at => "",
                    :updated_by => "admin"
                },

                {
                    :date => "2018-07-07T00:00:00.000+00:00",
                    :hours => random_fake_longterm_hours(),
                    :comment => "",
                    :created_at => "2018-07-19T08:04:00.000+00:00",
                    :created_by => "admin",
                    :updated_at => "",
                    :updated_by => "admin"
                },

                {
                    :date => "2018-07-08T00:00:00.000+00:00",
                    :hours => random_fake_longterm_hours(),
                    :comment => "",
                    :created_at => "2018-07-19T08:05:00.000+00:00",
                    :created_by => "admin",
                    :updated_at => "",
                    :updated_by => "admin"
                },

                {
                    :date => "2018-07-09T00:00:00.000+00:00",
                    :hours => random_fake_longterm_hours(),
                    :comment => "",
                    :created_at => "2018-07-19T08:05:00.000+00:00",
                    :created_by => "admin",
                    :updated_at => "",
                    :updated_by => "admin"
                },

                {
                    :date => "2018-07-10T00:00:00.000+00:00",
                    :hours => random_fake_longterm_hours(),
                    :comment => "Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet.",
                    :created_at => "2018-07-19T08:05:00.000+00:00",
                    :created_by => "admin",
                    :updated_at => "",
                    :updated_by => "admin"
                },

                {
                    :date => "2018-07-15T00:00:00.000+00:00",
                    :hours => random_fake_longterm_hours(),
                    :comment => "",
                    :created_at => "2018-07-18T16:21:00.000+00:00",
                    :created_by => "admin",
                    :updated_at => "",
                    :updated_by => "admin"
                },

                # Add one data entry for the next day.
                {
                    :date => Date.tomorrow.to_datetime.iso8601,
                    :hours => random_fake_longterm_hours(),
                    :comment => "",
                    :created_at => "2018-07-20T12:13:00.000+00:00",
                    :created_by => "admin",
                    :updated_at => "",
                    :updated_by => "admin"
                }
            ]

        fake_data = {
            :accountData => account_data,

            :accountBalanceToDate => account_data.inject(0) do |balance, data|
                # Sum all the hours for the dates before or equal to the given date.
                current_date = Date.parse( data[:date] )

                if current_date <= date
                    balance + data[:hours]
                else
                    balance
                end
            end,

            :accountBalanceEnd => account_data.inject(0) do |balance, data|
                # Sum all the hours for the end balance.
                balance + data[:hours]
            end
        }

        render create_json_response(:ok, THIS_SOURCE, NO_SOURCE_DETAILS, fake_data)
    end

    def random_fake_longterm_hours
        rand(-5..5)
    end
  end
end
