module ServiceLayerRest
  module ResponseHelper
    THIS_SOURCE = "Rails"
    NO_DATA = "Keine Daten"
    NO_SOURCE = "?"
    NO_SOURCE_DETAILS = "-"

    def extract_data_and_state_from_response_string(response_string)
      data = nil
      state = Rack::Utils.status_code(:internal_server_error)

      begin
        response = Oj.load(response_string)

        state = response[Constants::STATUS_KEY] if response[Constants::STATUS_KEY].present?
        data = response[Constants::DATA_KEY] if response[Constants::DATA_KEY].present? || response[Constants::DATA_KEY].in?([true, false])
      rescue Exception => e
        # Silently ignore errors
      end

      return data, state
    end

    def parse_response_string_for_render(response_string)
      status = Rack::Utils.status_code(:internal_server_error)
      source = NO_SOURCE
      source_details = NO_SOURCE_DETAILS
      data = NO_DATA

      begin
        response = Oj.load(response_string)

        status = response[Constants::STATUS_KEY] if response[Constants::STATUS_KEY].present?
        source = response[Constants::SOURCE_KEY] if response[Constants::SOURCE_KEY].present?
        source_details = response[Constants::SOURCE_DETAILS_KEY] if response[Constants::SOURCE_DETAILS_KEY].present?
        data = response[Constants::DATA_KEY] if response[Constants::DATA_KEY].present? || response[Constants::DATA_KEY].in?([true, false])
      rescue Exception => e
        return parse_exception_for_render e
      end

      create_json_response(status, source, source_details, data)
    end

    def parse_exception_for_render(e)
      print_backtrace e if (Rails.env == "development")
      create_json_error_response(e.message)
    end

    def bad_params_for_render(param_key)
      create_json_response(:bad_request, THIS_SOURCE, "EUVIS.Units.Exceptions.EBadParams", "Parameter für '#{param_key}' sind fehlerhaft.")
    end

    def empty_list_for_render(name_of_empty_list)
      create_json_response(:not_found, THIS_SOURCE, "EUVIS.Units.Exceptions.EListIsEmpty", "Die Liste der #{name_of_empty_list} ist leer.")
    end

    def create_json_error_response(message)
      create_json_response(:internal_server_error, THIS_SOURCE, "EUVIS.Units.Exceptions.EStatusCodeException", (message.nil? || !message.instance_of?(String)) ? "Unbekannter Fehler." : message)
    end

    def create_json_response(status, source, source_details, data)
      puts "Response => :#{Constants::STATUS_KEY} => #{status} / :#{Constants::SOURCE_KEY} => #{source} / :#{Constants::SOURCE_DETAILS_KEY} => #{source_details} / :#{Constants::DATA_KEY} => #{(data.class == Hash || data.class == Array) ? "..." : data}" if $debug_output

      {
        :json => {
          Constants::SOURCE_KEY.to_sym => source,
          Constants::SOURCE_DETAILS_KEY.to_sym => source_details,
          Constants::STATUS_KEY.to_sym => Rack::Utils.status_code(status),
          Constants::DATA_KEY.to_sym => data,
        },
        Constants::STATUS_KEY.to_sym => status,
      }
    end

    def check_params(string_array, int_array)
      return "" if string_array.empty? && int_array.empty?

      string_array.each_with_object(params) do |key, obj|
        return key if obj[key].blank?
      end

      int_array.each_with_object(params) do |key, obj|
        return key if obj[key] == nil || !(obj[key].is_a?(Integer) || (obj[key].is_a?(String) && obj[key].is_integer?)) || obj[key].to_i < 0
      end

      ""
    end

    def check_params_for_tlist(params, main_key, name_of_empty_list, allow_empty = false)
      [main_key].each_with_object(params) do |key, obj|
        return empty_list_for_render(name_of_empty_list) if obj[key][:items].nil? || (obj[key][:items].length == 0 && !allow_empty)

        return bad_params_for_render(key) if obj[key].blank?
        obj[key][:items].each do |item|
          item.each do |inner_key, inner_obj|
            return bad_params_for_render("#{key} - #{inner_key}") if inner_obj.blank?
          end
        end
        return bad_params_for_render("#{key} - sortType") if obj[key][:sortType].nil?
        return bad_params_for_render("#{key} - listHelper") if (obj[key][:listHelper].nil? || obj[key][:listHelper].blank?)

        return empty_list_for_render(name_of_empty_list) if (obj[key][:listHelper][0].to_i < 1 && !allow_empty)
        #TODO: Make me work ==>
        #return bad_params_for_render("#{key} - sortComparer") if obj[key][:sortComparer].nil?
      end

      true
    end

    def print_backtrace(e)
        puts "Error during processing: #{$!}"
        puts "Backtrace:\n\t#{e.backtrace.join("\n\t")}"
    end

    def start_timer
      $performance_output == true ? Time.now : nil
    end

    def stop_timer(time, text)
      puts("#{text} => #{Time.now - time}") if $performance_output && time != nil
    end
  end
end
